
class ENS160():
    INTDataDrdyEN = 1 << 1
    INTDataDrdyDIS = 0 << 1
    IntGprDrdyDIS = 0 << 3
    IntGprDrdyEN = 1 << 3

    ENS160_DATA_STATUS_REG   =  0x20
    ENS160_DATA_AQI_REG      =  0x21
    ENS160_DATA_TVOC_REG     =  0x22
    ENS160_DATA_ECO2_REG     =  0x24

    ENS160_STANDARD_MODE     =  0x02
    ENS160_IDLE_MODE         =  0x01
    ENS160_SLEEP_MODE        =  0x00

    def __init__(self, i2c_addr = 0x53):
        self.i2c = i2c
        self.i2c.init(freq=100000, sda=pin20, scl=pin19)
        self.i2c_addr = i2c_addr
        self.begin()
    
    def get_status(self):
        data = self.__read_reg(self.ENS160_DATA_STATUS_REG, 1)
        return ((data[0] >> 2) & 0b00000011)

    def get_aqi(self):
        data = self.__read_reg(self.ENS160_DATA_AQI_REG, 1)
        if data != []:
            return data[0]
        else:
            raise ValueError("read reg error")

    def get_tvoc(self):
        data = self.__read_reg(self.ENS160_DATA_TVOC_REG, 2)
        value = self.__concat_bytes(data)
        return value
    
    def get_eco2(self):
        data = self.__read_reg(self.ENS160_DATA_ECO2_REG, 2)
        value = self.__concat_bytes(data)
        return value
    
    def set_temp_hum(self, temp, humi):
        buf = bytearray(5)
        temp = int((temp + 273.15) * 64)
        rh = int(humi * 512)
        buf[0] = 0x13
        buf[1] = temp & 0xff
        buf[2] = (temp & 0xff00) >> 8
        buf[3] = rh & 0xff
        buf[4] = (rh & 0xff00) >> 8
        self.i2c.write(self.i2c_addr, buf)

    def set_pwr_mode(self, data):
        buf = bytearray(2)
        buf[0] = 0x10
        buf[1] = data
        self.i2c.write(self.i2c_addr, buf)
        sleep(20)

    def begin(self):
        data = self.__read_reg(0x00, 2)
        if 0x160 != self.__concat_bytes(data):
            raise ValueError("ERR_IC_VERSION")
        self.set_pwr_mode(0x02)
        self.__set_int_mode(0x00)

    def __set_int_mode(self, mode):
        mode |= self.INTDataDrdyEN | self.IntGprDrdyDIS 
        buf = bytearray(2)
        buf[0] = 0x11
        buf[1] = mode
        self.i2c.write(self.i2c_addr, buf)
        sleep(20)

    def __concat_bytes(self, data):
        return data[0] | (data[1] << 8)
    
    def __read_reg(self, reg, lens):
        buf = bytearray(1)
        buf[0] = reg
        self.i2c.write(self.i2c_addr, buf)
        return self.i2c.read(self.i2c_addr, lens)
